#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "tx/request.h"


#include "iixa.h"	// TODO delete and remove int hack

exec sql include sqlca;

static int verbose = 1;
/*
EXEC SQL DECLARE XEMP TABLE
	(EMPNO     integer NOT NULL PRIMARY KEY,
	ENAME     varchar(32));
*/
void dbOpenError() {
	userlogc_warn("dbOpenError\n");
}

static int isInTxn() {
    exec sql begin declare section;
        int trxflag;
    exec sql end declare section;
	exec sql inquire_sql (:trxflag = transaction);
	return trxflag != 0;
}

static int getError(char *errbuf, size_t sz) {
    exec sql begin declare section;
        char errmsg[101];
    exec sql end declare section;
    exec sql inquire_sql (:errmsg = ERRORTEXT);

	/*
	 * - defines for sqlcodes are defined in generr.h
	 * - the meaning of the codes in documented in SQLRef guide
	 */
	if (sqlca.sqlcode < 0) {
   		userlogc_debug("ing_access: %s", errmsg);

		if (errbuf)
			userlogc_snprintf(errbuf, sz, "%s", errmsg);
	} else if (sqlca.sqlcode > 0) {
   		userlogc_debug("ing_access: %d %s", sqlca.sqlcode, errmsg);
	}

	return sqlca.sqlcode;
}

static void
doSelect(int argempnum, char *argename, int *rcnt) {
	int validCsr = 0;
	exec sql begin declare section;
		int empno = argempnum;
		char ename[21];
	exec sql end declare section;

	exec sql declare empcsr cursor for
		select empno, ename
		from xemp
		where empno >= :empno;

	userlogc_debug("ing_access: doSelect opening cursor (inTx=%d) ...", isInTxn());
    exec sql open empcsr;

	if (sqlca.sqlcode != 0) {
		userlogc_debug("ing_access: doSelect cursor open error %d", getError(NULL, 0));
		return;
	}

	userlogc_debug("ing_access: doSelect selecting ...");
	while (1) {
		exec sql fetch empcsr into :empno, :ename;

		if (sqlca.sqlcode == 0) {
			userlogc_debug("ing_access: doSelect selected empno %d", empno);
			validCsr = 1;
			*rcnt += 1;
			if (verbose)
				userlogc_debug("%d, %s\n", empno, ename);
		} else {
			userlogc_debug("ing_access: doSelect selected %d records", *rcnt);
			break;
		}
	}

	userlogc_debug("ing_access: doSelect close_cursor isValid=%d", validCsr);
	if (validCsr)
		exec sql close empcsr;
	userlogc_debug("ing_access: doSelect cursor closed");
}

static void
doInsert(int argempnum, char *argename) {
	exec sql begin declare section;
		int empnum = argempnum;
		char *ename = argename;
	exec sql end declare section;

	userlogc_debug("ing_access: insert into XEMP (empno, ename) values (%d, %s) (inTx=%d) (sqlcode=%d)",
		empnum, ename, isInTxn(), sqlca.sqlcode);
	exec sql insert into XEMP (empno, ename) values (:empnum, :ename);
	userlogc_debug("ing_access: finished insert (inTx=%d) (sqlcode=%d)", isInTxn(), sqlca.sqlcode);
	getError(NULL,0);
}

static void
doUpdate(int argempnum, char *argename) {
	exec sql begin declare section;
		int empno = argempnum;
		char *ename = argename;
	exec sql end declare section;

	exec sql update XEMP set ename = :ename where empno = (:empno);
}

static void
doDelete(int argempnum, char *argename) {
int rcnt;
	exec sql begin declare section;
		int empno = argempnum;
	exec sql end declare section;

	userlogc_debug("ing_access: delete from XEMP where empno >= %d (inTx=%d)", empno, isInTxn());
	exec sql delete from XEMP where empno >= (:empno);
	userlogc_debug("ing_access: delete finished (inTx=%d)", isInTxn());
doSelect(empno, "", &rcnt);
userlogc_debug("ing_access: doSelect from %d after doDelete rcnt=%d (%d) (inTx=%d)",
		empno, rcnt, sqlca.sqlcode, isInTxn());
}

static void
doInvalid(int argempnum, char *argename) {
	exec sql begin declare section;
		int empno = argempnum;
	exec sql end declare section;

	userlogc_debug("delete from xXEMP where empno >= %d\n", empno);
	exec sql delete from xXEMP where empno >= (:empno);
}

long ing_xaflags() {
    return 0L;
}

#ifdef HACK
static int hack = 0;
static void xaopen() {
	if (hack) {
		int ret = iixa_switch.xa_open_entry((char *) "INGRES BTDB2 as BTDB2", 403, TMNOFLAGS);
		if (ret != XA_OK)
			userlogc("ing_access: xaopen INGRES BTDB2 failure: %d\n", ret);
		hack = 0;
	}
}
#endif

int ing_access(test_req_t *req, test_req_t *resp)
{
	exec sql begin declare section;
		char *dbname;
		char *dbname2 = "BTDB2";
	exec sql end declare section;

    int status = -1;
    int rcnt = 0;   // no of matching records
	int empno = (*req->data ? atoi(req->data) : 8000);

#ifdef HACK
xaopen();
#endif
	(resp->data)[0] = 0;
	dbname = req->db;

	userlogc_debug("ing_access: op=%c conn=%s (inTx=%d)", req->op, dbname, isInTxn());

	userlogc_debug("ing_access: set conn  (inTx=%d)", isInTxn());
	exec sql set connection :dbname;
	getError(NULL, 0);
	userlogc_debug("ing_access: conn set  (inTx=%d)", isInTxn());

	if (sqlca.sqlcode < 0) {
		userlogc_debug("ing_access: set conn error %d", getError(NULL, 0));
		resp->status = sqlca.sqlcode;
		return resp->status;
	}

	if (req->op == '0') {	/* insert */
		doInsert(empno, "Jim");
	} else if (req->op == '1') {	/* select */
		doSelect(empno, "", &rcnt);
		userlogc_snprintf(resp->data, sizeof (resp->data), "%d", rcnt);
		userlogc_debug("%d records (%s)\n", rcnt, resp->data);
	} else if (req->op == '2') {	/* update */
		doUpdate(empno, "Bob");
	} else if (req->op == '3') {	/* delete */
		doDelete(empno, "");
	} else {	/* ? */
		userlogc_debug("invalid request op: %d\n", req->op);
		/*doInvalid(empno, "");*/
	}

	status = getError(resp->data, sizeof (resp->data));
	userlogc_debug("ing_access: result=%d", status);

	userlogc_debug("ing_access: clear conn (inTx=%d)", isInTxn());
#ifdef HACK
	exec sql set connection :dbname2;
#else
	exec sql set connection NONE;
#endif
	getError(NULL, 0);
	userlogc_debug("ing_access: clear conn result: %d (inTx=%d)", sqlca.sqlcode, isInTxn());

	resp->status = (status < 0 ? status : 0);
	return resp->status;
}
